C+
      subroutine emdlv(r,vp,vs)
C
C  Replaces the routine with the same name and its entry, emdld, in the
C    IASPEI91 package.  This version allows introduction of other models
C    to prodouce tables.  REMODL calls emdld first.  Here it opens a file
C    containing the model (ascii).  The first two lines are comment lines
C    for the P and S models respectively.  The remaining lines are z, vp,
C    vs, density per line in free format starting from the surface down.
C    (Density is not used and could be left out.)  First order discontinuities
C    are included by repeated z values.  At this stage the user is prompted
C    for the model name and the filespec.  The calls to emdlv use cubic spline
C    interpolation to get the vp and vs for the desired r.  R0=6371.0 is 
C    assumed.  Limit is 200 layers and a maximum of 30 discontinuities.
C
C  Arthur Snoke  VTSO  5 April 1991
C
C  21 June 1992:  Changed interpolation from linear to cubic splines.
C-
      save
      parameter (max=200, npmax=30, max2=100)
      logical log,too_much
      character*(*) name
      character*80 filespec, dummy
      real*4 zin(max), vpin(max), vsin(max), rd(npmax),rdisc(npmax)
      real*4 a(max2,npmax,2),b(max2,npmax,2),c(max2,npmax,2)
      integer nlayer(npmax),ndisc(npmax)
c
      depth = AMAX1(6371.0 - r,0.0)
      TOO_MUCH = .FALSE.
      j = 1
      r = rzero - depth
      DO WHILE (.NOT.TOO_MUCH .AND. j .LE.ND)
        if (r .le. rdisc(j)) then
	  vp = speval(nlayer(j),depth,zin(ndisc(j)),vpin(ndisc(j)),
     1	    a(1,j,1),b(1,j,1),c(1,j,1))
	  vs = speval(nlayer(j),depth,zin(ndisc(j)),vsin(ndisc(j)),
     1	    a(1,j,2),b(1,j,2),c(1,j,2))
	  TOO_MUCH = .TRUE.
	else
	  j = j + 1
	end if
      END DO
      RETURN
C
      entry emdld(np,rd,name)
      call query('Model name..[return for IASP91]..:',log)
      read(*,'(a)') name
      if (lenc(name) .eq. 0) name = 'IASP91'
      call query('V-Z model filespec..:',LOG)
      read(*,'(A)') filespec
      CLOSE(UNIT=13)
      call assign(13,1,filespec)
      read(13,'(A)') dummy
      read(13,'(a)') dummy
C
      NIN=1
      IERR = 0
      DO WHILE ((IERR .EQ. 0) .AND. (NIN.LE.MAX))
C 
C  READ IN VELOCITY-DEPTH MODELS (BOTH P AND S) 
C
        READ(13,*,IOSTAT=IERR) ZIN(NIN), VPIN(NIN), VSIN(NIN)
        IF (IERR .EQ. 0) THEN
          NZ = NIN
          NIN = NIN + 1
        END IF
      END DO
      rzero = zin(nz)
      CLOSE(UNIT=13)
C
C      Now for the discontinuities
C
      np = 0
      do j=nz-1,2,-1
        if (zin(j) .eq. zin(j+1)) then
          np = np + 1
          rd(np) = rzero - zin(j)
	  ndisc(np) = j + 1
	  if (np .eq. 1) then
	    nlayer(1) = nz + 1 - ndisc(1)
	  else
	    nlayer(np) = ndisc(np-1) - ndisc(np)
	  end if
	  call spspln(nlayer(np),zin(ndisc(np)),vpin(ndisc(np)),
     1	    a(1,np,1),b(1,np,1),c(1,np,1))
	  call spspln(nlayer(np),zin(ndisc(np)),vsin(ndisc(np)),
     1	    a(1,np,2),b(1,np,2),c(1,np,2))
        end if
      end do
      np = np + 1
      rd(np) = rzero
      ndisc(np) = 1
      nlayer(np) = ndisc(np-1) - 1
      call spspln(nlayer(np),zin,vpin,
     1	    a(1,np,1),b(1,np,1),c(1,np,1))
      call spspln(nlayer(np),zin,vsin,
     1	    a(1,np,2),b(1,np,2),c(1,np,2))
	nd = np
	do j=1,nd
	  rdisc(j) = rd(j)
	end do
      RETURN
      end
	function lenc(string)
C+
C	function lenc(string)
C
C	Returns length of character variable STRING excluding right-hand
C	  most blanks or nulls
C-
	character*(*) string
	length = len(string)	! total length
	if (length .eq. 0) then
	  lenc = 0
	  return
	end if
	if(ichar(string(length:length)).eq.0)string(length:length) = ' '
	do j=length,1,-1
	  lenc = j
	  if (string(j:j).ne.' ' .and. ichar(string(j:j)).ne.0) return
	end do
	lenc = 0
	return
	end
C+
C     SUBROUTINE SPSPLN (N,X,Y,B,C,D) 
C 
C  THE COEFFICIENTS B(I), C(I) AND D(I) , I=1,2..N ARE COMPUTED 
C     FOR A CUBIC INTERPOLATING SPLINE 
C 
C  S(X) = X(I) + B(I)*(X-X(I)) + C(I)*(X-X(I))**2 + D(I)*(X-X(I)**3 
C 
C      FOR X(I) .LE. X .LE. X(I+1) 
C 
C  INPUT 
C 
C    N = THE NUMBER OF DATA POINTS  (N.GE.2) 
C    X = ABSCISSAS OF DATA POINTS IN STRICTLY INCREASING ORDER 
C    Y = ORDINATES OF DATA POINTS 
C 
C  OUTPUT 
C 
C    B, C, D ARE THE ARRAYS OF THE SPLINE COEFFICIENTS 
C 
C  THE FUNCTION SUBPROGRAM SPEVAL CAN BE USED TO EVALUATE THE SPLINE 
C	This is the single precision version
C	Reference: Forsythe et al., 1977 Ch. 4
C- 
      SUBROUTINE SPSPLN (N,X,Y,B,C,D)
      DIMENSION X(N),Y(N),B(N),C(N),D(N)
      NM1=N-1
         IF(N.LT.2) RETURN
         IF(N.LT.3) GO TO 50
      D(1)=X(2)-X(1)
      C(2)=(Y(2)-Y(1))/D(1)
      DO 10 I=2,NM1
         D(I)=X(I+1)-X(I)
         B(I)=2.0D0*(D(I-1)+D(I))
         C(I+1)=(Y(I+1)-Y(I))/D(I)
         C(I)=C(I+1)-C(I)
   10 CONTINUE
      B(1)=-D(1)
      B(N)=-D(N-1)
      C(1)=0.0D0
      C(N)=0.0D0
         IF(N.EQ.3) GO TO 15
      C(1)=C(3)/(X(4)-X(2))-C(2)/(X(3)-X(1))
      C(N)=C(N-1)/(X(N)-X(N-2))-C(N-2)/(X(N-1)-X(N-3))
      C(1)=C(1)*D(1)**2/(X(4)-X(1))
      C(N)=-C(N)*D(N-1)**2/(X(N)-X(N-3))
   15 DO 20 I=2,N
          T=D(I-1)/B(I-1)
          B(I)=B(I)-T*D(I-1)
          C(I)=C(I)-T*C(I-1)
   20 CONTINUE
      C(N)=C(N)/B(N)
      DO 30 IB=1,NM1
         I=N-IB
         C(I)=(C(I)-D(I)*C(I+1))/B(I)
   30 CONTINUE
      B(N)=(Y(N)-Y(NM1))/D(NM1)+D(NM1)*(C(NM1)+2.0D0*C(N))
      DO 40 I=1,NM1
         B(I)=(Y(I+1)-Y(I))/D(I)-D(I)*(C(I+1)+2.0D0*C(I))
         D(I)=(C(I+1)-C(I))/D(I)
         C(I)=3.0D0*C(I)
   40 CONTINUE
      C(N)=3.0D0*C(N)
      D(N)=D(N-1)
              RETURN
C
   50 B(1)=(Y(2)-Y(1))/(X(2)-X(1))
      C(1)=0.0D0
      D(1)=0.0D0
      B(2)=B(1)
      C(2)=0.0D0
      D(2)=0.0D0
         RETURN
      END
C+
C     FUNCTION SPEVAL(N,U,X,Y,B,C,D) 
C 
C  EVALUATES THE CUBIC SPLINE FUNCTION 
C 
C    SPEVAL = Y(I) + B(I)*(U-X(I)) + C(I)*(U-X(I)**2 + D(I)*(U-X(I))**3 
C 
C       WHERE X(I).LT.U.LT.X(I+1),  USING HORNER'S RULE 
C 
C  IF U.LT.X(1) THEN I=1 IS USED 
C  IF U.GE.X(N) THEN I=N IS USED 
C 
C  INPUT 
C      N=NUMBER OF DATA POINTS 
C      U=ABSCISSA AT WHICH SPLINE IS TO BE EVALUATED 
C      X,Y ARE ARRAYS OF DATA ABSCISSAS AND ORDINATES 
C      B,C,D ARE ARRAYS OF SPLINE COEFFICIENTS COMPUTED BY SPLINE 
C 
C  IF U IS NOT IN THE SAME INTERVAL AS THE PREVIOUS CALL, THEN A 
C      BINARY SEARCH IS PERFORMED TO DETERMINE PROPER INTERVAL 
C	This is the single-precision version of the program
C	Reference:  Forsythe et al., 1977 Ch. 4
C- 
      FUNCTION SPEVAL(N,U,X,Y,B,C,D)
      DIMENSION X(N),Y(N),B(N),C(N),D(N)
      DATA I/1/
         IF(I.GE.N) I=1
         IF(U.LT.X(I)) GO TO 10
         IF(U.LT.X(I+1)) GO TO 30
   10 I=1
      J=N+1
   20 K=(I+J)/2
         IF(U.LT.X(K)) J=K
         IF(U.GE.X(K)) I=K
         IF(J.GT.I+1) GO TO 20
   30 DX=U-X(I)
      SPEVAL=Y(I)+DX*(B(I)+DX*(C(I)+DX*D(I)))
         RETURN
      END
