C+
      program hypotable
C
C      Reads from .hed and .tbl files prepared by REMODL and SETBRN (of
C      the IASP91 package) and prepares ascii table files in the
C      April 1991 hypoellipse traveltime table format.
C      With one exception, I have used the conventions of the IASP91
C      package.  The exception is that I added a VAX "assign" routine so
C      I could open files readonly for read access and carriagecontrol=
C      LIST for write files. 
C  Arthur Snoke  Virginia Tech 6 June 1991
C	7 February 2000: changed crossover from horizontal ray
C		to downgoing (Pn) comes in earlier than upward (Pg)
C	9 February 2000:  Increased the number of v-z layers to be 
C		included in the table to 20, in accordance with Lahr.
C	10 April 2001:  Conversions to kilometers should be done for a
C		spherical earth with a radius of 6371 km.  Took out
C		conversions to kilometers based on a local radius.
C	29 April 2002: Used the (hidden) iasp91 method to return the
C		velocity at a fixed depth rather than calculating it
C		by interpolating the v-z model.  Also, the reducing
C		velocity was changed to the final velocity used rather
C		than the final velocity in the input model.
C-
      save
      parameter (max=60)
      logical prnt(3),log, more
      character*1 phlst
      character*8 phcd(max)
      character*40 modnam
	external F
      dimension tt(max),dtdd(max),dtdh(max),dddp(max)
      dimension usrc(2)
      data in/1/,prnt/2*.false.,.true./
      CHARACTER*20 COMMNT
      CHARACTER*80 filespec
      DIMENSION DIST(42),NTABLE(42),TABLE(42),DEPTHT(28),dtdz(42)
      DIMENSION V(200),Z(200)
      DATA RDDEG,MAX1/57.29578,200/
	common /iasp/ phcd,tokm
      common /rzero/ rzero
	common /numfun/ numfun
      DATA tol/0.00001/,rzero/6371.0/
C
      call vassign(2,3,'hypotable.lst')
      modnam = ' '
      call tabin(in,modnam)
      write(2,*) 'Model Name: ',modnam
      TOKM = rzero/RDDEG
      call query('Type of wave: Enter P or S',log)
      read(*,'(A1)') phlst
      if (phlst .eq. 'P') then
        ips = 1
      else
        ips = 2
      end if
      call brnset(1,phlst,prnt)
C  START RUN
C  CALL VELOCITY MODELS.   K=1,2 FOR P MODEL, S MODEL
      CALL VZMOD(V,Z,NVZ,MAX1,3,2,IPS)
C
C	The maximum number of v-z entries allowed by Hypoellipse is 20
C
      if (nvz .gt. 20) write(*,*) 'There are',nvz,
     1       ' layers.  Only 20 in final model.'
      nvzmax = min0(20,nvz)
      call query('Filespec for output table:',log)
      read(*,'(A)') filespec
      nf = lenc(filespec)
      call vassign(8,4,filespec(1:nf))
      write(2,*) '  Table file: ',filespec(1:nf)
      call query('Comment about model (20A1)',log)
      read(*,'(a)') commnt
      nc = lenc(commnt)
      write(2,*) commnt(1:nc)
      IF (NC .LT. 20) COMMNT(NC+1:20) = ' '
C
C  Traveltimes are calculated at depths of 0.0 and NZ1 values at increments
C  of DZ1, then NZ2 values at increments of DZ2. This permits a fine-grid
C  spacing for shallow depths and a coarse spacing at greater depths where
C  the travel-time curve will be smoother. Similarly, travel times are
C  calculated at distances of 0.0, DD1, 2DD1, up to ND1 * DD1, and then at
C  ND2 values in increments of DD2. Presently the maximum value of NZ1 + NZ2
C  is 27, and ND1 + ND2 may be as large as 41.
C
      DEPTHT(1) = 0.0
      WRITE(*,*) 'Up to 28 depths in two ranges starting from 0 km'
      call query('Number of depth increments in first set',log)
      read(*,*) nz1
      nz1 = min0(27,nz1)
      call query('Maximum depth for first set',log)
      read(*,*) depmax
      DZ1 = DEPMAX/NZ1
      DO J=1,NZ1
        DEPTHT(J+1) = J*DZ1
      END DO
      WRITE(2,*) 'NZ1 =',NZ1,'  DEPMAX1 =',DEPMAX,'  DZ1 =',DZ1
      IF (NZ1 .LT. 27) THEN
        X = FLOAT(27 - NZ1)
        Call query('Number of depth increments in second set',log)
        read(*,*) nz2
        nz2 = min0(nz2,27-nz1)
        IF (NZ2 .GT. 0) THEN
          call query('Maximum depth for second set',log)
          read(*,*) depmax
          DZ2 = (DEPMAX-DEPTHT(NZ1+1))/NZ2
          do j=1,NZ2
            DEPTHT(J+NZ1+1) = DEPTHT(NZ1+1) + J*DZ2
          END DO
          WRITE(2,*) 'NZ2 =',NZ2,'  DEPMAX2 =',DEPMAX,'  DZ2 =',DZ2
        ELSE
          DZ2 = 0.0
          WRITE(2,*) 'NZ2 =',NZ2,'   DZ2 =',DZ2
        END IF
      ELSE
        NZ2 = 0
        DZ2 = 0.0
        WRITE(2,*) 'NZ2 =',NZ2,'   DZ2 =',DZ2
      END IF
      NDEPTH = 1 + NZ1 + NZ2
      WRITE(*,*) 'Up to 42 distances at each depth in two ranges'
      WRITE(*,*) '     starting from 0 km'
      call query('# distance increments in first set',log)
      read(*,*) nd1
      nd1 = min0(41,nd1)
      call query('Maximum distance for first set',log)
      read(*,*) dmax1
      DD1 = DMAX1/ND1
      DIST(1) = 0.0
      DO J=1,ND1
        DIST(J+1) = J*DD1
      END DO
      WRITE(2,*) 'ND1 =',ND1,'  DMAX1 =',DMAX1,'  DD1 =',DD1
      IF (ND1 .LT. 41) THEN
        X = FLOAT(41 - ND1)
        call query('Number of dist. incr. in second set',log)
        read(*,*) nd2
        nd2 = min0(nd2,41-nd1)
        IF (ND2 .GT. 0) THEN
          call query('Maximum distance',log)
          read(*,*) dmax
          DD2 = (DMAX-DMAX1)/ND2
          DO J=1,ND2
            DIST(J+1+ND1) = DMAX1 + J*DD2
          END DO
          WRITE(2,*) 'ND2 =',ND2,'  DMAX =',DMAX,'  DD2 =',DD2
        ELSE
          DD2 = 0
          DMAX = DMAX1
          WRITE(2,*) 'ND2 =',ND2,'   DD2 =',DD2
        END IF
      ELSE
        ND2 = 0
        DD2 = 0.0
        DMAX = DMAX1
        WRITE(2,*) 'ND2 =',ND2,'   DD2 =',DD2
      END IF
      NDIST = 1 + ND1 + ND2
      REDVEL = 1.0/V(nvzmax-1)
      call query('Velocity Ratio:  VP/VS',log)
      read(*,*) vpvs
      write(2,*) 'Reducing velocity is',REDVEL
      write(2,*) 'VP/VS is',vpvs
      WRITE(8,'(A20,I5,2F10.5,F10.3)') COMMNT,NVZMAX,REDVEL,
     1          vpvs,rzero
      WRITE(8,'('' Z '',15F7.2)') (Z(K),K=1,NVZMAX)
      WRITE(8,'('' V '',15F7.2)') (V(K),K=1,NVZMAX)
      WRITE(8,'(''DD '',2(F10.4,I5))') DD1,ND1,DD2,ND2
      WRITE(8,'(''DZ '',2(F10.4,I5))') DZ1,NZ1,DZ2,NZ2
      do jd=1,ndepth
        DEPTH = DEPTHT(jd)
        call depset(depth,usrc)
C
C	usrc returns the P and S slownesses -- transformed to flat-earth
C	(28 Sept 2001) If only P or S, may get returned as zero 
C
	if (ips .eq. 1) then
      	  vfoc = ((rzero-depth)/rzero)/usrc(1)
	else
	  vfoc = ((rzero-depth)/rzero)/usrc(2)
	end if
c
        DO J=1,NDIST
          delta = dist(j)/tokm
          call trtm(delta,max,n,tt,dtdd,dtdh,dddp,phcd)
          if(n.le.0) then
            write(*,*)'No arrival: depth',depth,' and distance',dist(j)
            stop
          else
            TABLE(J) = tt(1)
            NTABLE(J) = NINT(2000*(TABLE(J)-REDVEL*DIST(J))) - 32000
	    dtdz(j) = -dtdh(1)
          end if
        END DO
C
C	The table includes the epicentral distance in units of 0.1 km 
C	at which the first-arrival rays change from upgoing to downgoing.
C	This may not be the same as the Pg-Pn crossover, becasue a Pn may
C	exist for some distances and not be the first arrival.  Also, a
C	downgoing Pg may arrive before the Pn (e.g., surface focus for
C	which there is no upgoing or horizontal ray beyond an epicentral
C	distance of 0 km).  We call this distance cross_dist.
C
	if (jd .eq. 1) then
	  cross_dist = 0.0
	else
	  more = .true.
	  jj = 2
	  do while (more .and. (jj.lt.ndist))
		if (dtdz(jj) .lt. 0.0) then
			jj = jj + 1
		else
			more = .false.
		end if
	  end do
	  cross_dist = zeroin(dist(jj-1),dist(jj),F,tol,numfun)  
	end if
	jcross_dist = nint(10.0*cross_dist)
c	if (jd .gt. 1) write(2,*) depth,cross_dist,dist(jj-1),
c     1	dtdz(jj-1),dist(jj),dtdz(jj),numfun
        WRITE(8,'(2F10.4,I10)') depth,vfoc,jcross_dist
        WRITE(8,'(15I8)') (NTABLE(J),J=1,NDIST)
      end do
      call exit(0)
      END
C  lenc.for
C+
      function lenc(string)
C
C      Returns length of character variable STRING excluding right-hand
C        most blanks.  Also stops shor of a tab.
C-
      character*(*) string
      character*1 tab/9/
      length = len(string)      ! total length
      if (length .eq. 0) then
        lenc = 0
        return
      end if
      ntab = index(string(1:length),tab)
      if (ntab .gt. 0) length = ntab - 1
      if (length .eq. 0) then
        lenc = 0
        return
      end if
      do j=length,1,-1
        lenc = j
        if (string(j:j) .ne. ' ') return
      end do
      lenc = 0
      return
      end
c  end lenc.for
c vassign.for
c+
      subroutine vassign(lun,mod,filename)
c
c   Opens files.  May be machine dependent (sun conventions)
C  lun is the logical unit number
C  mod=1 for existing files (does not safeguard for readonly)
C  mod=2 for new files (cannot already exist)
C  mod=any thing else for status unknown)
C  filename is the name of the file
C  assumed to be formatted and sequential
c  no error checking
c-
      character*(*) filename
      close(unit=lun)
      if (mod .eq. 1) then
        open(unit=lun,name=filename,status='old')
      else if (mod .eq. 2) then
        open(unit=lun,name=filename,status='new')
      else
        open(unit=lun,name=filename,status='unknown')
      end if
      return
      end
c  end vassign.for
C  VZMOD.for
C+
      SUBROUTINE VZMOD(V,Z,N,MAX,IFILE,JFILE,IPS)
C 
C  READS AND RETURNS A P OR S MODEL
C  IFILE IS THE INPUT FILE NUMBER, JFILE the OUTPUT
C  (IF JFILE > 0, WRITES MODEL TO THAT FILE)
C  IPS IS 1 OR 2 FOR A P OR S MODEL
C- 
      LOGICAL LOG,REPEAT
      CHARACTER*125 VZCOM,FILESPEC
      DIMENSION V(*),Z(*),TEMPV(2) 
C
CC      Get file
C
      call query('V-Z model filespec',LOG)
      read(*,'(A)') filespec
      nf = lenc(filespec)
      CLOSE(UNIT=IFILE)
      call vassign(ifile,1,filespec(1:nf))
      read(ifile,'(A)') vzcom
      if (ips .eq. 2) then
        read(ifile,'(a)') vzcom
      else
        read(ifile,*)
      end if
      nc = lenc(vzcom)
C
      NIN=1
      REPEAT = .TRUE.
      DO WHILE (REPEAT .AND. NIN.LE.MAX)
C 
C  READ IN VELOCITY-DEPTH MODELS (BOTH P AND S) 
C
        READ(IFILE,*,IOSTAT=IERR) ZIN,TEMPV
        IF (IERR .EQ. 0 .AND. TEMPV(IPS) .GT. 0.0) THEN
          N = NIN
          Z(N) = ZIN
          V(N)=TEMPV(IPS)
        ELSE
          REPEAT = .FALSE.
        END IF
        NIN=NIN+1
      END DO
      CLOSE(UNIT=IFILE)
      if (jfile .le. 0) return
C
      write(jfile,*) 'V-Z model from file ',filespec(1:nf)
      WRITE(jfile,*) VZCOM(1:nc)
      write(jfile,*)
      WRITE(jfile,'(4(5X,''Depth'',2X,''Velocity''))')
      LINES=N/4+1 
      LINE1=N-3*LINES 
      LINE2=LINE1+1 
      IF (LINE1 .EQ. 0) LINE1=LINES
      DO K=1,LINE1 
        NMAX=K+3*LINES 
        WRITE(jfile,'(4(F10.1,F10.4))') (Z(KN),V(KN),KN=K,NMAX,LINES) 
      END DO
      IF(LINE1 .LT. LINES) THEN
        DO K=LINE2,LINES 
          NMAX=K+2*LINES 
          WRITE(jfile,'(4(F10.1,F10.4))') (Z(KN),V(KN),KN=K,NMAX,LINES) 
        END DO
      END IF
      WRITE(jfile,*)
      RETURN 
      END 
C  end vzmod.for
C  subroutine zeroin.f
C+
      FUNCTION ZEROIN(AX,BX,F,TOL,NUMFUN)
C
C  A ZERO IN THE FUNCTION F(X) IS COMPUTED IN THE INTERVAL AX,BX
C
C  INPUT
C
C  AX LEFT ENDPOINT OF INITIAL INTERVAL
C  BX RIGHT ENDPOINT OF INITIAL INTERVAL
C  F  FUNCTION SUBPROGRAM WHICH EVALUATES F(X) FOR ANY X IN
C        THE INTERVAL AX,BX
C  TOL   DESIRED LENGTH OF THE INTERVAL OF UNCERTAINTY OF THE
C        FINAL RESULT (.GE.0.0)
C
C
C  OUTPUT
C
C  ZEROIN ABCISSA APPROXIMATING A ZERO OF F IN THE INTERVAL AX,BX
C  NUMFUN IS THE NUMBER OF EVALUATIONS OF F(X)
C
C
C     IT IS ASSUMED THAT F(AX) AND F(BX) HAVE OPPOSITE SIGNS
C  WITHOUT A CHECK  ZEROIN RETRUNS A ZERO X IN THE GIVEN INTERVAL
C  AX,BX TO WITHIN A TOLERANCE 4*MACHEPS*ABS(X) + TOL WHERE MACHEPS
C  IS THE RELATIVE MACHINE PRECISION
C     THIS FUNCTION SUBPROGRAM IS A SLIGHTLY MODIFIED TRANSLATION OF
C  THE ALGOL 60 PROCEDURE ZERO GIVEN IN RICHARD BRENT ALGORITHMS FOR
C  MINIMIZATION WITHOUT DERIVATIVES PRENTICE - HALL, INC. (1973)
C
C	June 1991 JAS/VTSO changed check.  Input compares with machine
C	  precision:  output with TOL.
C-
C
C  COMPUTE EPS THE RELATIVE MACHINE PRECISION
C
      EPS = 1.0
   10 EPS = EPS/2
      TOL1 = 1.0 + EPS
      IF (TOL1 .GT. 1.0) GO TO 10
C
C  INITIALIZATION
C
      A = AX
      B = BX
      NUMFUN = 1
      FA = F(A)
      NUMFUN = 2
      FB = F(B)
C
C  BEGIN STEP
C
   20 C = A
      FC = FA
      D = B - A
      E = D
   30 IF (ABS(FC) .GE. ABS(FB)) GO TO 40
      A = B
      B = C
      C = A
      FA = FB
      FB = FC
      FC = FA
C
C  CONVERGENCE TEST
C
   40 TOL1 = 2.0*EPS*ABS(B)
      XM = 0.5*(C - B)
      IF (ABS(XM) .LE. TOL1 .or. abs(fb) .le. 0.5*tol) GO TO 90
C
C  IS BISECTION NECESSARY?
C
      IF (ABS(E) .LT. TOL1) GO TO 70
      IF (ABS(FA) .LE. ABS(FB)) GO TO 70
C
C  IS QUADRATIC INTERPOLATION POSSIBLE?
C
      IF (A .NE. C) GO TO 50
C
C  LINEAR INTERPOLATION
C
      S = FB/FA
      P = 2.0*XM*S
      Q = 1.0 - S
      GO TO 60
C
C  INVERSE QUADRATIC INTERPOLATION
C
   50 Q = FA/FC
      R = FB/FC
      S = FB/FA
      P = S*(2.0*XM*Q*(Q - R) - (B - A)*(R - 1.0))
      Q = (Q - 1.0)*(R - 1.0)*(S - 1.0)
C
C  ADJUST SIGNS
C
   60 IF (P .GT. 0.0) Q =-Q
      P = ABS(P)
C
C  IS INTERPOLATION ACCEPTABLE?
C
      IF ((2.0*P) .GE. (3.0*XM*Q - ABS(TOL1)*Q)) GO TO 70
      IF (P .GE. ABS(0.5*E*Q)) GO TO 70
      E = D
      D = P/Q
      GO TO 80
C
C  BISECTION
C
   70 D = XM
      E = D
C
C  COMPLETE STEP
C
   80 A = B
      FA = FB
      IF (ABS(D) .GT. TOL1) B = B + D
      IF (ABS(D) .LE. TOL1) B = B + SIGN(TOL1,XM)
      NUMFUN = NUMFUN + 1
      FB = F(B)
      IF ((FB*(FC/ABS(FC))) .GT. 0) GO TO 20
      GO TO 30
C
C  DONE
C
   90 ZEROIN = B
      RETURN
      END
c  end zeroin.f
c  function F
	Function F(X)
	parameter (max=60)
	character*8 phcd(max)
	common /numfun/ numfun
	dimension tt(max),dtdd(max),dtdh(max),dddp(max)
	common /iasp/ phcd,tokm
        delta = x/tokm
        call trtm(delta,max,n,tt,dtdd,dtdh,dddp,phcd)
C
C	dtdh has the opposite sign from dtdz.  We know that the lowest
C	distance has positive dtdz (times increasing as depth increases)
C	and maximum distance has negative dtdh
	if (numfun .le. 8) then
C
C	This allows for the possibility that the first arrival can be
C	a downgoing Pg and not Pn
C
	  F = -dtdh(1)
	else
C
C	This allows for the fact that Pn exists short of the crossover
C	and Pg exists at greater distances
C
	  F = sign(1.0,dtdh(1))*(tt(1)-tt(2))
	end if
c	write(2,*) numfun,dtdh(1),delta,tt(1),tt(2)
	return
	end
